@@ -18,7 +18,7 @@ from coherence.upnp.core.DIDLLite import classChooser, Container, Resource, DIDL
 from coherence.upnp.core.soap_proxy import SOAPProxy
 from coherence.upnp.core.soap_service import errorCode
 
-import louie
+import coherence.extern.louie as louie
 
 from coherence.extern.simple_plugin import Plugin
 
