@@ -11,7 +11,7 @@ import shutil
 import time
 import re
 from datetime import datetime
-import urllib2
+import urllib
 
 import mimetypes
 mimetypes.init()
@@ -35,7 +35,7 @@ from coherence.extern.inotify import IN_CHANGED
 
 from coherence.extern.xdg import xdg_content
 
-import louie
+import coherence.extern.louie as louie
 
 from coherence.backend import BackendItem, BackendStore
 
@@ -108,7 +108,7 @@ class FSItem(BackendItem):
                 size = 0
 
             if mimetype != 'item':
-                res = Resource('file://'+ urllib2.quote(self.get_path()), 'internal:%s:%s:*' % (host,self.mimetype))
+                res = Resource('file://'+ urllib.quote(self.get_path()), 'internal:%s:%s:*' % (host,self.mimetype))
                 res.size = size
                 self.item.res.append(res)
 
@@ -162,7 +162,7 @@ class FSItem(BackendItem):
                         self.item.res.append(new_res)
                         if not hasattr(self.item, 'attachments'):
                             self.item.attachments = {}
-                        self.item.attachments[hash_from_path] = utils.StaticFile(urllib2.quote(thumbnail))
+                        self.item.attachments[hash_from_path] = utils.StaticFile(urllib.quote(thumbnail))
 
 
             try:
@@ -195,7 +195,7 @@ class FSItem(BackendItem):
         else:
             host = host_port
 
-        res = Resource('file://'+urllib2.quote(self.get_path()), 'internal:%s:%s:*' % (host,self.mimetype))
+        res = Resource('file://'+urllib.quote(self.get_path()), 'internal:%s:%s:*' % (host,self.mimetype))
         try:
             res.size = self.location.getsize()
         except:
@@ -217,10 +217,6 @@ class FSItem(BackendItem):
 
         self.parent.update_id += 1
 
-    def __del__(self):
-        #print "FSItem __del__", self.id, self.get_name()
-        pass
-
     def check_for_cover_art(self):
         """ let's try to find in the current directory some jpg file,
             or png if the jpg search fails, and take the first one
@@ -361,6 +357,9 @@ class FSStore(BackendStore):
         except:
             self.inotify = None
 
+        if kwargs.get('enable_destroy','no') == 'yes':
+            self.upnp_DestroyObject = self.hidden_upnp_DestroyObject
+
         self.ignore_file_pattern = re.compile('|'.join(['^\..*'] + list(ignore_patterns)))
         parent = None
         self.update_id = 0
@@ -387,6 +386,10 @@ class FSStore(BackendStore):
     def __repr__(self):
         return str(self.__class__).split('.')[-1]
 
+    def release(self):
+        if self.inotify != None:
+            self.inotify.release()
+
     def len(self):
         return len(self.store)
 
@@ -438,7 +441,7 @@ class FSStore(BackendStore):
                 self.warning("UnicodeDecodeError - there is something wrong with a file located in %r", container.get_path())
 
     def create(self, mimetype, path, parent):
-        #print "create", mimetype, path, type(path)
+        #print "create", mimetype, path, type(path), parent
         UPnPClass = classChooser(mimetype)
         if UPnPClass == None:
             return None
@@ -451,6 +454,7 @@ class FSStore(BackendStore):
         self.store[id] = FSItem( id, parent, path, mimetype, self.urlbase, UPnPClass, update=True)
         if hasattr(self, 'update_id'):
             self.update_id += 1
+            #print self.update_id
             if self.server:
                 if hasattr(self.server,'content_directory_server'):
                     self.server.content_directory_server.set_variable(0, 'SystemUpdateID', self.update_id)
@@ -463,7 +467,7 @@ class FSStore(BackendStore):
         return id
 
     def append(self,path,parent):
-        #print "append", path, type(path)
+        #print "append", path, type(path), parent
         try:
             mimetype,_ = mimetypes.guess_type(path, strict=False)
             if mimetype == None:
@@ -676,6 +680,23 @@ class FSStore(BackendStore):
 
         return failure.Failure(errorCode(712))
 
+    def hidden_upnp_DestroyObject(self, *args, **kwargs):
+        ObjectID = kwargs['ObjectID']
+
+        item = self.get_by_id(ObjectID)
+        if item == None:
+            return failure.Failure(errorCode(701))
+
+        print "upnp_DestroyObject", item.location
+        try:
+            item.location.remove()
+        except Exception, msg:
+            print Exception, msg
+            return failure.Failure(errorCode(715))
+
+        return {}
+
+
 if __name__ == '__main__':
 
     from twisted.internet import reactor
