@@ -17,7 +17,7 @@ import pygst
 pygst.require('0.10')
 import gst
 
-import louie
+import coherence.extern.louie as louie
 
 from coherence.extern.simple_plugin import Plugin
 
@@ -607,10 +607,22 @@ class GStreamerPlayer(log.Loggable,Plugin):
                              'http-get:*:audio/mp4:*',
                              'internal:%s:application/ogg:*' % self.server.coherence.hostname,
                              'http-get:*:application/ogg:*',
+                             'internal:%s:audio/ogg:*' % self.server.coherence.hostname,
+                             'http-get:*:audio/ogg:*',
+                             'internal:%s:video/ogg:*' % self.server.coherence.hostname,
+                             'http-get:*:video/ogg:*',
                              'internal:%s:video/x-msvideo:*' % self.server.coherence.hostname,
                              'http-get:*:video/x-msvideo:*',
+                             'internal:%s:video/mp4:*' % self.server.coherence.hostname,
+                             'http-get:*:video/mp4:*',
                              'internal:%s:video/quicktime:*' % self.server.coherence.hostname,
-                             'http-get:*:video/quicktime:*'],
+                             'http-get:*:video/quicktime:*',
+                             'internal:%s:image/gif:*' % self.server.coherence.hostname,
+                             'http-get:*:image/gif:*',
+                             'internal:%s:image/jpeg:*' % self.server.coherence.hostname,
+                             'http-get:*:image/jpeg:*',
+                             'internal:%s:image/png:*' % self.server.coherence.hostname,
+                             'http-get:*:image/png:*'],
                             default=True)
         self.server.av_transport_server.set_variable(0, 'TransportState', 'NO_MEDIA_PRESENT', default=True)
         self.server.av_transport_server.set_variable(0, 'TransportStatus', 'OK', default=True)
