@@ -50,7 +50,7 @@ from urlparse import urlsplit
 from axiom import store, item, attributes
 from epsilon.extime import Time
 
-import louie
+import coherence.extern.louie as louie
 
 from twisted.internet import reactor, defer
 
@@ -134,6 +134,8 @@ def sanitize(filename):
 
 class Container(BackendItem):
 
+    get_path = None
+
     def __init__(self, id, parent_id, name, children_callback=None):
         self.id = id
         self.parent_id = parent_id
@@ -186,6 +188,8 @@ class Artist(item.Item,BackendItem):
     name = attributes.text(allowNone=False, indexed=True)
     musicbrainz_id = attributes.text()
 
+    get_path = None
+
     def get_children(self,start=0,request_count=0):
         all_id = 'artist_all_tracks_%d' % (self.storeID+1000)
         self.store.containers[all_id] = \
@@ -233,6 +237,8 @@ class Album(item.Item,BackendItem):
     cd_count = attributes.integer(default=1)
     cover = attributes.text(default=u'')
 
+    get_path = None
+
     def get_children(self,start=0,request_count=0):
         children = list(self.store.query(Track, Track.album == self,sort=Track.track_nr.ascending))
         if request_count == 0:
@@ -379,6 +385,8 @@ class Playlist(item.Item,BackendItem):
     name = attributes.text(allowNone=False, indexed=True)
     # references to tracks
 
+    get_path = None
+
 class MediaStore(BackendStore):
     logCategory = 'media_store'
     implements = ['MediaServer']
@@ -543,19 +551,19 @@ class MediaStore(BackendStore):
 
             if self.coverlocation is not None:
                 cover_path = os.path.join(self.coverlocation,filename +'.jpg')
-            if os.path.exists(cover_path) is True:
-                print "cover found:", cover_path
-                album.cover = cover_path
-            else:
-                def got_it(f,a):
-                    print "cover saved:",f, a.title
-                    a.cover = f
+                if os.path.exists(cover_path) is True:
+                    print "cover found:", cover_path
+                    album.cover = cover_path
+                else:
+                    def got_it(f,a):
+                        print "cover saved:",f, a.title
+                        a.cover = f
 
-                aws_key = '1XHSE4FQJ0RK0X3S9WR2'
-                CoverGetter(cover_path,aws_key,
-                            callback=(got_it,(album)),
-                            artist=album.artist.name,
-                            title=album.title)
+                    aws_key = '1XHSE4FQJ0RK0X3S9WR2'
+                    CoverGetter(cover_path,aws_key,
+                                callback=(got_it,(album)),
+                                artist=album.artist.name,
+                                title=album.title)
 
     def get_by_id(self,id):
         self.info("get_by_id %s" % id)
