@@ -24,8 +24,10 @@ BUS_NAME = 'org.Coherence'     # the one with the dots
 OBJECT_PATH = '/org/Coherence'  # the one with the slashes ;-)
 
 from coherence import __version__
+from coherence.upnp.core import DIDLLite
 
-import louie
+
+import coherence.extern.louie as louie
 
 from coherence import log
 
@@ -50,16 +52,69 @@ class DBusService(dbus.service.Object,log.Loggable):
 
         self.subscribe()
 
+        #interfaces = self._dbus_class_table[self.__class__.__module__ + '.' + self.__class__.__name__]
+        #for (name, funcs) in interfaces.iteritems():
+        #    print name, funcs
+        #    if funcs.has_key('destroy_object'):
+        #        print """removing 'destroy_object'"""
+        #        del funcs['destroy_object']
+        #    for func in funcs.values():
+        #        if getattr(func, '_dbus_is_method', False):
+        #            print self.__class__._reflect_on_method(func)
+
+        #self._get_service_methods()
+
+    def _get_service_methods(self):
+        '''Returns a list of method descriptors for this object'''
+        methods = []
+        for func in dir(self):
+            func = getattr(self,func)
+            if callable(func) and hasattr(func, '_dbus_is_method'):
+                print func, func._dbus_interface, func._dbus_is_method
+                if hasattr(func, 'im_func'):
+                    print func.im_func
+
+    def path(self):
+        return OBJECT_PATH + '/devices/' + dbus_device.id + '/services/' + self.type
+
     def variable_changed(self,variable):
         #print self.service, "got signal for change of", variable
         #print variable.name, variable.value
         #print type(variable.name), type(variable.value)
-        self.StateVariableChanged(variable.name, variable.value)
+        self.StateVariableChanged(self.dbus_device.device.get_id(),self.type,variable.name, variable.value)
 
     @dbus.service.signal(BUS_NAME+'.service',
-                         signature='sv')
-    def StateVariableChanged(self, variable, value):
-        self.info("%s service %s signals StateVariable %s changed to >%s<" % (self.dbus_device.device.get_friendly_name(), self.type, variable, value))
+                         signature='sssv')
+    def StateVariableChanged(self, udn, service, variable, value):
+        self.info("%s service %s signals StateVariable %s changed to %r" % (self.dbus_device.device.get_friendly_name(), self.type, variable, value))
+
+    @dbus.service.method(BUS_NAME+'.service',in_signature='',out_signature='as')
+    def get_available_actions(self):
+        actions = self.service.get_actions()
+        r = []
+        for name in actions.keys():
+            r.append(name)
+        return r
+
+    @dbus.service.method(BUS_NAME+'.service',in_signature='sv',out_signature='v',
+                         async_callbacks=('dbus_async_cb', 'dbus_async_err_cb'))
+    def action(self,name,arguments,dbus_async_cb,dbus_async_err_cb):
+
+        def reply(data):
+            dbus_async_cb(dbus.Dictionary(data,signature='sv',variant_level=4))
+
+        if self.service.client is not None:
+            #print "action", name
+            func = getattr(self.service.client,name,None)
+            #print "action", func
+            if callable(func):
+                kwargs = {}
+                for k,v in arguments.items():
+                    kwargs[str(k)] = str(v)
+                d = func(**kwargs)
+                d.addCallback(reply)
+                d.addErrback(dbus_async_err_cb)
+        return ''
 
     @dbus.service.method(BUS_NAME+'.service',in_signature='v',out_signature='v',
                          async_callbacks=('dbus_async_cb', 'dbus_async_err_cb'))
@@ -77,7 +132,23 @@ class DBusService(dbus.service.Object,log.Loggable):
             d.addErrback(dbus_async_err_cb)
         return ''
 
-    @dbus.service.method(BUS_NAME+'.service',in_signature='',out_signature='v')
+    @dbus.service.method(BUS_NAME+'.service',in_signature='v',out_signature='v',
+                         async_callbacks=('dbus_async_cb', 'dbus_async_err_cb'))
+    def destroy_object(self,arguments,dbus_async_cb,dbus_async_err_cb):
+
+        def reply(data):
+            dbus_async_cb(dbus.Dictionary(data,signature='sv',variant_level=4))
+
+        if self.service.client is not None:
+            kwargs = {}
+            for k,v in arguments.items():
+                kwargs[str(k)] = str(v)
+            d = self.service.client.destroy_object(**kwargs)
+            d.addCallback(reply)
+            d.addErrback(dbus_async_err_cb)
+        return ''
+
+    @dbus.service.method(BUS_NAME+'.service',in_signature='',out_signature='ssv')
     def subscribe(self):
         notify = [v for v in self.service._variables[0].values() if v.send_events == True]
         if len(notify) == 0:
@@ -104,7 +175,7 @@ class DBusService(dbus.service.Object,log.Loggable):
                     data[unicode(n.name)] = lc
             else:
                 data[unicode(n.name)] = unicode(n.value)
-        return dbus.Dictionary(data,signature='sv',variant_level=3)
+        return self.dbus_device.device.get_id(), self.type, dbus.Dictionary(data,signature='sv',variant_level=3)
 
 
 class DBusDevice(dbus.service.Object,log.Loggable):
@@ -114,19 +185,23 @@ class DBusDevice(dbus.service.Object,log.Loggable):
         self.device = device
         self.id = device.get_id()[5:].replace('-','')
         bus_name = dbus.service.BusName(BUS_NAME+'.device', bus)
-        d = dbus.service.Object.__init__(self, bus_name, OBJECT_PATH + '/devices/' + self.id)
+        d = dbus.service.Object.__init__(self, bus_name, self.path())
         self.debug("DBusDevice %r %r %r", device, self.id, d)
         self.services = []
         for service in device.get_services():
             self.services.append(DBusService(service,self,bus))
 
+    def path(self):
+        return OBJECT_PATH + '/devices/' + self.id
+
     @dbus.service.method(BUS_NAME+'.device',in_signature='',out_signature='v')
     def get_info(self):
-        r = [self.device.get_device_type(),
-                self.device.get_friendly_name(),
-                self.device.get_usn(),
-                self.services]
-        return dbus.Array(r,signature='v',variant_level=2)
+        r = {'path':self.path(),
+             'device_type':self.device.get_device_type(),
+             'friendly_name':self.device.get_friendly_name(),
+             'udn':self.device.get_id(),
+             'services':self.services}
+        return dbus.Dictionary(r,signature='sv',variant_level=2)
 
     @dbus.service.method(BUS_NAME+'.device',in_signature='',out_signature='s')
     def get_friendly_name(self):
@@ -144,6 +219,11 @@ class DBusDevice(dbus.service.Object,log.Loggable):
     def get_usn(self):
         return self.device.get_usn()
 
+    @dbus.service.method(BUS_NAME+'.device',in_signature='',out_signature='av')
+    def get_device_icons(self):
+        return dbus.Array(self.device.icons,signature='av',variant_level=2)
+
+
 class DBusPontoon(dbus.service.Object,log.Loggable):
     logCategory = 'dbus'
 
@@ -161,9 +241,9 @@ class DBusPontoon(dbus.service.Object,log.Loggable):
             self.devices.append(DBusDevice(device,self.bus_name))
 
         louie.connect(self.cp_ms_detected, 'Coherence.UPnP.ControlPoint.MediaServer.detected', louie.Any)
-        #louie.connect(self.UPnP_ControlPoint_MediaServer_removed, 'Coherence.UPnP.ControlPoint.MediaServer.removed', louie.Any)
+        louie.connect(self.cp_ms_removed, 'Coherence.UPnP.ControlPoint.MediaServer.removed', louie.Any)
         louie.connect(self.cp_mr_detected, 'Coherence.UPnP.ControlPoint.MediaRenderer.detected', louie.Any)
-        #louie.connect(self.UPnP_ControlPoint_MediaRenderer_removed, 'Coherence.UPnP.ControlPoint.MediaRenderer.removed', louie.Any)
+        louie.connect(self.cp_mr_removed, 'Coherence.UPnP.ControlPoint.MediaRenderer.removed', louie.Any)
         louie.connect(self.remove_client, 'Coherence.UPnP.Device.remove_client', louie.Any)
 
     def remove_client(self, usn, client):
@@ -173,23 +253,33 @@ class DBusPontoon(dbus.service.Object,log.Loggable):
         except:
             pass
 
+    def remove(self,udn):
+        #print "DBusPontoon remove", udn
+        for device in self.devices:
+            #print "check against", device.device.get_id()
+            if udn == device.device.get_id():
+                device.device = None
+                self.devices.remove(device)
+                break
+
     @dbus.service.method(BUS_NAME,in_signature='',out_signature='s')
     def version(self):
         return __version__
 
-    @dbus.service.method(BUS_NAME,in_signature='',out_signature='as')
+    @dbus.service.method(BUS_NAME,in_signature='',out_signature='av')
     def get_devices(self):
         r = []
         for device in self.devices:
-            r.append(device.get_id())
-        return r
+            #r.append(device.path())
+            r.append(device.get_info())
+        return dbus.Array(r,signature='v',variant_level=2)
 
     @dbus.service.method(BUS_NAME,in_signature='s',out_signature='v')
     def get_device_with_id(self,id):
         r = {}
         device = self.controlpoint.get_device_with_id(id)
         r['id'] = device.get_id()
-        r['usn'] = device.usn
+        r['udn'] = device.udn
         r['name'] = device.get_friendly_name()
         r['type'] = device.get_device_type()
         return r
@@ -199,30 +289,53 @@ class DBusPontoon(dbus.service.Object,log.Loggable):
         id = "n/a"
         return id
 
-    def cp_ms_detected(self,client,usn=''):
-        self.devices.append(DBusDevice(client.device,self.bus))
-        self.UPnP_ControlPoint_MediaServer_detected(usn)
+    @dbus.service.method(BUS_NAME,in_signature='sa{ss}',out_signature='s')
+    def add_plugin(self,backend,arguments):
+        kwargs = {}
+        for k,v in arguments.iteritems():
+            kwargs[str(k)] = unicode(v)
+        p = self.controlpoint.coherence.add_plugin(backend,**kwargs)
+        return str(p.uuid)
 
-    def cp_mr_detected(self,client,usn=''):
-        self.devices.append(DBusDevice(client.device,self.bus))
-        self.UPnP_ControlPoint_MediaRenderer_detected(usn)
+    @dbus.service.method(BUS_NAME,in_signature='s',out_signature='s')
+    def remove_plugin(self,uuid):
+        return self.controlpoint.coherence.remove_plugin(uuid)
+
+    def cp_ms_detected(self,client,udn=''):
+        new_device = DBusDevice(client.device,self.bus)
+        self.devices.append(new_device)
+        self.UPnP_ControlPoint_MediaServer_detected(new_device.get_info(),udn)
+
+    def cp_mr_detected(self,client,udn=''):
+        new_device = DBusDevice(client.device,self.bus)
+        self.devices.append(new_device)
+        self.UPnP_ControlPoint_MediaRenderer_detected(new_device.get_info(),udn)
+
+    def cp_ms_removed(self,udn):
+        #print "cp_ms_removed", udn
+        self.UPnP_ControlPoint_MediaServer_removed(udn)
+        self.remove(udn)
+
+    def cp_mr_removed(self,udn):
+        self.UPnP_ControlPoint_MediaRenderer_removed(udn)
+        self.remove(udn)
 
     @dbus.service.signal(BUS_NAME,
-                         signature='s')
-    def UPnP_ControlPoint_MediaServer_detected(self,usn):
+                         signature='vs')
+    def UPnP_ControlPoint_MediaServer_detected(self,device,udn):
         self.info("emitting signal UPnP_ControlPoint_MediaServer_detected")
 
     @dbus.service.signal(BUS_NAME,
                          signature='s')
-    def UPnP_ControlPoint_MediaServer_removed(self,usn):
+    def UPnP_ControlPoint_MediaServer_removed(self,udn):
         self.info("emitting signal UPnP_ControlPoint_MediaServer_removed")
 
     @dbus.service.signal(BUS_NAME,
-                         signature='s')
-    def UPnP_ControlPoint_MediaRenderer_detected(self,usn):
+                         signature='vs')
+    def UPnP_ControlPoint_MediaRenderer_detected(self,device,udn):
         self.info("emitting signal UPnP_ControlPoint_MediaRenderer_detected")
 
     @dbus.service.signal(BUS_NAME,
                          signature='s')
-    def UPnP_ControlPoint_MediaRenderer_removed(self,usn):
+    def UPnP_ControlPoint_MediaRenderer_removed(self,udn):
         self.info("emitting signal UPnP_ControlPoint_MediaRenderer_removed")
