@@ -9,13 +9,13 @@ from urlparse import urlsplit
 
 from twisted.internet import reactor, defer
 from twisted.web import resource, server
-from twisted.internet.protocol import Protocol, ClientCreator
+from twisted.internet.protocol import Protocol, ClientCreator, _InstanceFactory
 from twisted.python import failure
 
 from coherence import log, SERVER_ID
 from coherence.upnp.core import utils
 
-import louie
+import coherence.extern.louie as louie
 
 global hostname, web_server_port
 hostname = None
@@ -125,6 +125,8 @@ class EventSubscriptionServer(resource.Resource, log.Loggable):
                 #print headers['sid']
                 if self.subscribers.has_key(headers['sid']):
                     s = self.subscribers[headers['sid']]
+                    s['timeout'] = headers['timeout']
+                    s['created'] = time.time()
                 elif not headers.has_key('callback'):
                     request.setResponseCode(404)
                     request.setHeader('SERVER', SERVER_ID)
@@ -136,10 +138,9 @@ class EventSubscriptionServer(resource.Resource, log.Loggable):
                 s = { 'sid' : str(sid),
                       'callback' : headers['callback'][1:len(headers['callback'])-1],
                       'seq' : 0}
-                reactor.callLater(0.8, self.service.new_subscriber, s)
-
-            s['timeout'] = headers['timeout']
-            s['created'] = time.time()
+                s['timeout'] = headers['timeout']
+                s['created'] = time.time()
+                self.service.new_subscriber(s)
 
             request.setHeader('SID', s['sid'])
             #request.setHeader('Subscription-ID', sid)  wrong example in the UPnP UUID spec?
@@ -190,10 +191,18 @@ class EventProtocol(Protocol, log.Loggable):
         self.service = service
         self.action = action
 
+    def teardown(self):
+        self.transport.loseConnection()
+        self.service.event_connection = None
+
     def connectionMade(self):
-        self.timeout_checker = reactor.callLater(30, lambda : self.transport.loseConnection())
+        self.timeout_checker = reactor.callLater(30, self.teardown)
 
     def dataReceived(self, data):
+        try:
+            self.timeout_checker.cancel()
+        except:
+            pass
         self.info("response received from the Service Events HTTP server ")
         #self.debug(data)
         cmd, headers = utils.parse_http_response(data)
@@ -209,12 +218,11 @@ class EventProtocol(Protocol, log.Loggable):
                     self.service.set_timeout(time.time() + 4294967296) # FIXME: that's lame
                 elif timeout.startswith('Second-'):
                     timeout = int(timeout[len('Second-'):])
-                    self.service.set_timeout(time.time() + timeout)
+                    self.service.set_timeout(timeout)
             except:
                 #print headers
                 pass
-        self.transport.loseConnection()
-
+        self.teardown()
 
     def connectionLost( self, reason):
         try:
@@ -247,9 +255,12 @@ def subscribe(service, action='subscribe'):
         log.info(log_category, "event.subscribe.send_request %r, action: %r %r",
                  p, action, service.get_event_sub_url())
         if action == 'subscribe':
+            timeout = service.timeout
+            if timeout == 0:
+                timeout = 1800
             request = ["SUBSCRIBE %s HTTP/1.1" % service.get_event_sub_url(),
                         "HOST: %s:%d" % (host, port),
-                        "TIMEOUT: Second-1800",
+                        "TIMEOUT: Second-%d" % timeout,
                         ]
             service.event_connection = p
         else:
@@ -326,6 +337,10 @@ class NotificationProtocol(Protocol, log.Loggable):
         self.timeout_checker = reactor.callLater(30, lambda : self.transport.loseConnection())
 
     def dataReceived(self, data):
+        try:
+            self.timeout_checker.cancel()
+        except:
+            pass
         cmd, headers = utils.parse_http_response(data)
         self.debug( "notification response received %r %r", cmd, headers)
         try:
@@ -360,7 +375,7 @@ def send_notification(s, xml):
         host = host_port
         port = 80
 
-    def send_request(p):
+    def send_request(p,port_item):
         request = ['NOTIFY %s HTTP/1.1' % path,
                     'HOST:  %s:%d' % (host, port),
                     'SEQ:  %d' % s['seq'],
@@ -380,14 +395,23 @@ def send_notification(s, xml):
         if s['seq'] > 0xffffffff:
             s['seq'] = 1
         p.transport.write(request)
+        port_item.disconnect()
         #return p.transport.write(request)
 
-    def got_error(failure):
+    def got_error(failure,port_item):
+        port_item.disconnect()
         log.info(log_category, "error sending notification to %r %r",
                  s['sid'], s['callback'])
         log.debug(log_category, failure)
 
-    c = ClientCreator(reactor, NotificationProtocol)
-    d = c.connectTCP(host, port)
-    d.addCallback(send_request)
-    d.addErrback(got_error)
+    #c = ClientCreator(reactor, NotificationProtocol)
+    #d = c.connectTCP(host, port)
+
+    d = defer.Deferred()
+    f = _InstanceFactory(reactor, NotificationProtocol(), d)
+    port_item = reactor.connectTCP(host, port, f, timeout=30, bindAddress=None)
+
+    d.addCallback(send_request, port_item)
+    d.addErrback(got_error, port_item)
+
+    return d,port_item
