@@ -5,6 +5,7 @@
 # Copyright 2006, Frank Scholz <coherence@beebits.net>
 
 import socket
+import time
 
 from twisted.internet.protocol import DatagramProtocol
 from twisted.internet import reactor
@@ -12,7 +13,7 @@ from twisted.internet import task
 
 from coherence.upnp.core import utils
 
-import louie
+import coherence.extern.louie as louie
 
 SSDP_PORT = 1900
 SSDP_ADDR = '239.255.255.250'
@@ -22,12 +23,13 @@ from coherence import log
 class MSearch(DatagramProtocol, log.Loggable):
     logCategory = 'msearch'
 
-    def __init__(self, ssdp_server):
+    def __init__(self, ssdp_server, test=False):
         self.ssdp_server = ssdp_server
-        port = reactor.listenUDP(0, self)
+        if test == False:
+            self.port = reactor.listenUDP(0, self)
 
-        l = task.LoopingCall(self.double_discover)
-        l.start(120.0)
+            self.double_discover_loop = task.LoopingCall(self.double_discover)
+            self.double_discover_loop.start(120.0)
 
     def datagramReceived(self, data, (host, port)):
         cmd, headers = utils.parse_http_response(data)
@@ -42,6 +44,9 @@ class MSearch(DatagramProtocol, log.Loggable):
                                             headers['server'],
                                             headers['cache-control'],
                                             host=host)
+            else:
+                self.ssdp_server.known[headers['usn']]['last-seen'] = time.time()
+                self.debug('updating last-seen for %r' % headers['usn'])
 
         # make raw data available
         # send out the signal after we had a chance to register the device
