@@ -20,11 +20,11 @@ from coherence.upnp.core.event import EventSubscriptionServer
 from coherence.extern.et import ET
 
 from twisted.web import static
-from twisted.internet import defer
+from twisted.internet import defer, reactor
 from twisted.python import failure, util
 from twisted.internet import task
 
-import louie
+import coherence.extern.louie as louie
 
 from coherence import log
 
@@ -93,6 +93,12 @@ class Service(log.Loggable):
 
     def remove(self):
         self.info("removal of ", self.device.friendly_name, self.service_type, self.id)
+        try:
+            self.renew_subscription_call.cancel()
+        except:
+            pass
+        if self.event_connection != None:
+            self.event_connection.teardown()
         if self.subscription_id != None:
             self.unsubscribe()
         for name,action in self._actions.items():
@@ -115,7 +121,15 @@ class Service(log.Loggable):
         return self.service_type
 
     def set_timeout(self, timeout):
+        self.info("set timout for %s/%s to %d" %(self.device.friendly_name,self.service_type,int(timeout)))
         self.timeout = timeout
+        try:
+            self.renew_subscription_call.reset(int(self.timeout)-30)
+            self.info("reset renew subscription call for %s/%s to %d" % (self.device.friendly_name,self.service_type,int(self.timeout)-30))
+        except:
+            self.renew_subscription_call = reactor.callLater(int(self.timeout)-30,
+                                                                 self.renew_subscription)
+            self.info("starting renew subscription call for %s/%s to %d" % (self.device.friendly_name,self.service_type,int(self.timeout)-30))
 
     def get_timeout(self):
         return self.timeout
@@ -127,6 +141,7 @@ class Service(log.Loggable):
         return self.subscription_id
 
     def set_sid(self, sid):
+        self.info("set subscription id for %s/%s to %s" %(self.device.friendly_name,self.service_type,sid))
         self.subscription_id = sid
         if sid is not None:
             subscribe(self)
@@ -136,14 +151,16 @@ class Service(log.Loggable):
         return self._actions
 
     def get_action( self, name):
-        return self.get_actions()[name]
+        try:
+            return self._actions[name]
+        except KeyError:
+            return None # not implemented
 
     def get_state_variables(self, instance):
-        return self._variables.get(instance)
+        return self._variables.get(int(instance))
 
     def get_state_variable(self, name, instance=0):
-        instance = int(instance)
-        return self._variables.get(instance).get(name)
+        return self._variables.get(int(instance)).get(name)
 
     def get_control_url(self):
         return self.url_base + self.control_url
@@ -194,6 +211,7 @@ class Service(log.Loggable):
 
 
     def renew_subscription(self):
+        self.info("renew_subscription")
         event.subscribe(self)
 
     def process_event(self,event):
@@ -213,6 +231,27 @@ class Service(log.Loggable):
                         self.info("%r %r %r" % (namespace_uri, tag,var.attrib['val']))
                         self.get_state_variable(tag, instance_id).update(var.attrib['val'])
                         self.info("updated var %r" % var)
+                        if len(var.attrib) > 1:
+                            self.info("Extended StateVariable %s - %r", var.tag, var.attrib)
+                            if var.attrib.has_key('channel') and var.attrib['channel'] != 'Master':
+                                # TODO handle attributes that them selves have multiple instances
+                                self.info("Skipping update to %s its not for master channel %s", var.tag, var.attrib)
+                                pass
+                            else:
+                                if not self.get_state_variables(instance_id):
+                                    # TODO Create instance ?
+                                    self.error("%r update failed (not self.get_state_variables(instance_id)) %r", self, instance_id)
+                                elif not self.get_state_variables(instance_id).has_key(tag):
+                                    # TODO Create instance StateVariable?
+                                    # SONOS stuff
+                                    self.error("%r update failed (not self.get_state_variables(instance_id).has_key(tag)) %r", self, tag)
+                                else:
+                                    val = None
+                                    if var.attrib.has_key('val'):
+                                        val = var.attrib['val']
+                                    #self.debug("%r update %r %r %r", self,namespace_uri, tag, var.attrib['val'])
+                                    self.get_state_variable(tag, instance_id).update(var.attrib['val'])
+                                    self.debug("updated 'attributed' var %r", var)
             else:
                 self.get_state_variable(var_name, 0).update(var_value)
         if self.last_time_updated == None:
@@ -325,6 +364,8 @@ class ServiceServer(log.Loggable):
         self._variables = {0: {}}
         self._subscribers = {}
 
+        self._pending_notifications = {}
+
         self.last_change = None
         self.init_var_and_actions()
 
@@ -339,13 +380,22 @@ class ServiceServer(log.Loggable):
         self.check_subscribers_loop = task.LoopingCall(self.check_subscribers)
         self.check_subscribers_loop.start(120.0, now=False)
 
+        self.check_moderated_loop = None
         if moderated_variables.has_key(self.service_type):
             self.check_moderated_loop = task.LoopingCall(self.check_moderated_variables)
             #self.check_moderated_loop.start(5.0, now=False)
             self.check_moderated_loop.start(0.5, now=False)
 
+    def _release(self):
+        for p in self._pending_notifications.values():
+            p.disconnect()
+        self._pending_notifications = {}
+
     def get_action(self, action_name):
-        return self._actions[action_name]
+        try:
+            return self._actions[action_name]
+        except KeyError:
+            return None # not implemented
 
     def get_actions(self):
         return self._actions
@@ -356,6 +406,9 @@ class ServiceServer(log.Loggable):
     def get_subscribers(self):
         return self._subscribers
 
+    def rm_notification(self,result,d):
+        del self._pending_notifications[d]
+
     def new_subscriber(self, subscriber):
         notify = []
         for vdict in self._variables.values():
@@ -384,7 +437,9 @@ class ServiceServer(log.Loggable):
 
         if evented_variables > 0:
             xml = ET.tostring( root, encoding='utf-8')
-            event.send_notification(subscriber, xml)
+            d,p = event.send_notification(subscriber, xml)
+            self._pending_notifications[d] = p
+            d.addBoth(self.rm_notification,d)
         self._subscribers[subscriber['sid']] = subscriber
 
     def get_id(self):
@@ -426,10 +481,11 @@ class ServiceServer(log.Loggable):
                 len(self._subscribers) > 0):
                 xml = self.build_single_notification(instance, variable_name, variable.value)
                 for s in self._subscribers.values():
-                    event.send_notification(s, xml)
-
+                    d,p = event.send_notification(s, xml)
+                    self._pending_notifications[d] = p
+                    d.addBoth(self.rm_notification,d)
         try:
-            variable = self._variables[instance][variable_name]
+            variable = self._variables[int(instance)][variable_name]
             if isinstance( value, defer.Deferred):
                 value.addCallback(process_value)
             else:
@@ -439,7 +495,7 @@ class ServiceServer(log.Loggable):
 
     def get_variable(self, variable_name, instance=0):
         try:
-            return self._variables[instance][variable_name]
+            return self._variables[int(instance)][variable_name]
         except:
             return None
 
@@ -509,7 +565,9 @@ class ServiceServer(log.Loggable):
         xml = ET.tostring( root, encoding='utf-8')
         #print "propagate_notification", xml
         for s in self._subscribers.values():
-            event.send_notification(s, xml)
+            d,p = event.send_notification(s,xml)
+            self._pending_notifications[d] = p
+            d.addBoth(self.rm_notification,d)
 
     def check_subscribers(self):
         for s in self._subscribers.values():
@@ -555,10 +613,15 @@ class ServiceServer(log.Loggable):
         for action_node in tree.findall('.//action'):
             name = action_node.findtext('name')
             implementation = 'required'
+            needs_callback = False
             if action_node.find('Optional') != None:
                 implementation = 'optional'
+                if action_node.find('Optional').attrib.get(
+                                        '{urn:schemas-beebits-net:service-1-0}X_needs_backend',
+                                        None) != None:
+                    needs_callback = True
+
             arguments = []
-            needs_callback = False
             for argument in action_node.findall('.//argument'):
                 arg_name = argument.findtext('name')
                 arg_direction = argument.findtext('direction')
