@@ -4,7 +4,7 @@
 # Copyright 2007 - Frank Scholz <coherence@beebits.net>
 
 from twisted.web import server, resource
-from twisted.python import log, failure
+from twisted.python import failure
 from twisted.internet import defer
 
 from coherence import log, SERVER_ID
@@ -15,7 +15,7 @@ from coherence.upnp.core.utils import parse_xml
 
 from coherence.upnp.core import soap_lite
 
-import louie
+import coherence.extern.louie as louie
 
 class errorCode(Exception):
     def __init__(self, status):
@@ -142,6 +142,9 @@ class UPnPPublisher(resource.Resource, log.Loggable):
             if(headers.has_key('user-agent') and
                     headers['user-agent'].find('Xbox/') == 0):
                 keywords['X_UPnPClient'] = 'XBox'
+            if(headers.has_key('user-agent') and
+                    headers['user-agent'].startswith("""Mozilla/4.0 (compatible; UPnP/1.0; Windows""")):
+                keywords['X_UPnPClient'] = 'XBox'
             if(headers.has_key('x-av-client-info') and
                     headers['x-av-client-info'].find('"PLAYSTATION3') > 0):
                 keywords['X_UPnPClient'] = 'PLAYSTATION3'
