@@ -21,7 +21,7 @@ from twisted.internet import task
 
 from coherence import log, SERVER_ID
 
-import louie
+import coherence.extern.louie as louie
 
 SSDP_PORT = 1900
 SSDP_ADDR = '239.255.255.250'
@@ -35,26 +35,40 @@ class SSDPServer(DatagramProtocol, log.Loggable):
 
     _callbacks = {}
 
-    def __init__(self):
-
+    def __init__(self,test=False):
         # Create SSDP server
-        try:
-            port = reactor.listenMulticast(SSDP_PORT, self, listenMultiple=True)
-            port.setLoopbackMode(1)
+        self.test = test
+        if self.test == False:
+            try:
+                self.port = reactor.listenMulticast(SSDP_PORT, self, listenMultiple=True)
+                #self.port.setLoopbackMode(1)
 
-            port.joinGroup(SSDP_ADDR)
+                self.port.joinGroup(SSDP_ADDR)
 
-            l = task.LoopingCall(self.resendNotify)
-            l.start(777.0, now=False)
+                self.resend_notify_loop = task.LoopingCall(self.resendNotify)
+                self.resend_notify_loop.start(777.0, now=False)
 
-        except error.CannotListenError, err:
-            self.warning("There seems to be already a SSDP server running on this host, no need starting a second one.")
+                self.check_valid_loop = task.LoopingCall(self.check_valid)
+                self.check_valid_loop.start(333.0, now=False)
+
+            except error.CannotListenError, err:
+                self.warning("There seems to be already a SSDP server running on this host, no need starting a second one.")
+
+        self.active_calls = []
 
     def shutdown(self):
+        for call in reactor.getDelayedCalls():
+            if call.func == self.send_it:
+                call.cancel()
+        if self.resend_notify_loop.running:
+            self.resend_notify_loop.stop()
+        if self.check_valid_loop.running:
+            self.check_valid_loop.stop()
         '''Make sure we send out the byebye notifications.'''
-        for st in self.known:
-            if self.known[st]['MANIFESTATION'] == 'local':
-                self.doByebye(st)
+        if self.test == False:
+            for st in self.known:
+                if self.known[st]['MANIFESTATION'] == 'local':
+                    self.doByebye(st)
 
     def datagramReceived(self, data, (host, port)):
         """Handle a received multicast datagram."""
@@ -110,6 +124,7 @@ class SSDPServer(DatagramProtocol, log.Loggable):
         self.known[usn]['MANIFESTATION'] = manifestation
         self.known[usn]['SILENT'] = silent
         self.known[usn]['HOST'] = host
+        self.known[usn]['last-seen'] = time.time()
 
         self.msg(self.known[usn])
 
@@ -140,7 +155,10 @@ class SSDPServer(DatagramProtocol, log.Loggable):
         self.debug('Notification headers:', headers)
 
         if headers['nts'] == 'ssdp:alive':
-            if not self.isKnown(headers['usn']):
+            try:
+                self.known[headers['usn']]['last-seen'] = time.time()
+                self.debug('updating last-seen for %r' % headers['usn'])
+            except KeyError:
                 self.register('remote', headers['usn'], headers['nt'], headers['location'],
                               headers['server'], headers['cache-control'], host=host)
         elif headers['nts'] == 'ssdp:byebye':
@@ -150,6 +168,13 @@ class SSDPServer(DatagramProtocol, log.Loggable):
             self.warning('Unknown subtype %s for notification type %s' %
                     (headers['nts'], headers['nt']))
 
+    def send_it(self,response,destination,delay,usn):
+        self.info('send discovery response delayed by %ds for %s to %r' % (delay,usn,destination))
+        try:
+            self.transport.write(response,destination)
+        except (AttributeError,socket.error), msg:
+            self.info("failure sending out byebye notification: %r" % msg)
+
     def discoveryRequest(self, headers, (host, port)):
         """Process a discovery request.  The response must be sent to
         the address specified by (host, port)."""
@@ -157,13 +182,6 @@ class SSDPServer(DatagramProtocol, log.Loggable):
         self.info('Discovery request from (%s,%d) for %s' % (host, port, headers['st']))
         self.info('Discovery request for %s' % headers['st'])
 
-        def send_it(response, destination, delay, usn):
-            self.info('send discovery response delayed by %ds for %s to %r' % (delay, usn, destination))
-            try:
-                self.transport.write(response, destination)
-            except (AttributeError,socket.error), msg:
-                self.info("failure sending out byebye notification: %r" % msg)
-
         # Do we know about this service?
         for i in self.known.values():
             if i['MANIFESTATION'] == 'remote':
@@ -185,7 +203,8 @@ class SSDPServer(DatagramProtocol, log.Loggable):
 
                 response.extend(('', ''))
                 delay = random.randint(0, int(headers['mx']))
-                reactor.callLater(delay, send_it,
+
+                reactor.callLater(delay, self.send_it,
                                 '\r\n'.join(response), (host, port), delay, usn)
 
     def doNotify(self, usn):
@@ -205,6 +224,8 @@ class SSDPServer(DatagramProtocol, log.Loggable):
         del stcpy['MANIFESTATION']
         del stcpy['SILENT']
         del stcpy['HOST']
+        del stcpy['last-seen']
+
         resp.extend(map(lambda x: ': '.join(x), stcpy.iteritems()))
         resp.extend(('', ''))
         self.debug('doNotify content', resp)
@@ -230,6 +251,7 @@ class SSDPServer(DatagramProtocol, log.Loggable):
             del stcpy['MANIFESTATION']
             del stcpy['SILENT']
             del stcpy['HOST']
+            del stcpy['last-seen']
             resp.extend(map(lambda x: ': '.join(x), stcpy.iteritems()))
             resp.extend(('', ''))
             self.debug('doByebye content', resp)
@@ -246,6 +268,28 @@ class SSDPServer(DatagramProtocol, log.Loggable):
             if self.known[usn]['MANIFESTATION'] == 'local':
                 self.doNotify(usn)
 
+    def check_valid(self):
+        """ check if the discovered devices are still ok, or
+            if we haven't received a new discovery response
+        """
+        self.debug("Checking devices/services are still valid")
+        removable = []
+        for usn in self.known:
+            if self.known[usn]['MANIFESTATION'] != 'local':
+                _,expiry = self.known[usn]['CACHE-CONTROL'].split('=')
+                expiry = int(expiry)
+                now = time.time()
+                last_seen = self.known[usn]['last-seen']
+                self.debug("Checking if %r is still valid - last seen %d (+%d), now %d" % (self.known[usn]['USN'],last_seen,expiry,now))
+                if last_seen + expiry + 30 < now:
+                    self.debug("Expiring: %r" % self.known[usn])
+                    if self.known[usn]['ST'] == 'upnp:rootdevice':
+                        louie.send('Coherence.UPnP.SSDP.removed_device', None, device_type=self.known[usn]['ST'], infos=self.known[usn])
+                    removable.append(usn)
+        while len(removable) > 0:
+            usn = removable.pop(0)
+            del self.known[usn]
+
     def subscribe(self, name, callback):
         self._callbacks.setdefault(name,[]).append(callback)
 
