@@ -8,12 +8,20 @@ import urlparse
 
 from coherence.extern.et import parse_xml as et_parse_xml
 
+from coherence import SERVER_ID
+
+
 from twisted.web import server, http, static
 from twisted.web import client, error
 from twisted.web import proxy, resource, server
-from twisted.internet import reactor, protocol
+from twisted.internet import reactor,protocol,defer
 from twisted.python import failure
 
+try:
+    from twisted.protocols._c_urlarg import unquote
+except ImportError:
+    from urllib import unquote
+
 try:
     import netifaces
     have_netifaces = True
@@ -102,7 +110,7 @@ def get_host_address():
     try:
         route_file = '/proc/net/route'
         route = open(route_file)
-        if (route):
+        if(route):
             tmp = route.readline() #skip first line
             while (tmp != ''):
                 tmp = route.readline()
@@ -111,7 +119,7 @@ def get_host_address():
                     if l[2] != '00000000': #default gateway...
                         route.close()
                         return get_ip_address(l[0])
-    except IOerror:
+    except IOerror, msg:
         """ fallback to parsing the output of netstat """
         from os import uname
         import posix
@@ -127,6 +135,9 @@ def get_host_address():
                     return get_ip_address(parts[5])
                 else:
                     return get_ip_address(parts[-1])
+    except Exception, msg:
+        import traceback
+        traceback.print_exc()
 
     """ return localhost if we haven't found anything """
     return '127.0.0.1'
@@ -156,13 +167,44 @@ def de_chunk_payload(response):
 
     return newresponse.getvalue()
 
+class Request(server.Request):
+
+    def process(self):
+        "Process a request."
+
+        # get site from channel
+        self.site = self.channel.site
+
+        # set various default headers
+        self.setHeader('server', SERVER_ID)
+        self.setHeader('date', http.datetimeToString())
+        self.setHeader('content-type', "text/html")
+
+        # Resource Identification
+        self.prepath = []
+        self.postpath = map(unquote, self.path[1:].split('/'))
+        try:
+            def deferred_rendering(r):
+                self.render(r)
+
+            resrc = self.site.getResourceFor(self)
+            if isinstance(resrc, defer.Deferred):
+                resrc.addCallback(deferred_rendering)
+                resrc.addErrback(self.processingFailed)
+            else:
+                self.render(resrc)
+        except:
+            self.processingFailed(failure.Failure())
+
 
 class Site(server.Site):
 
     noisy = False
+    requestFactory = Request
 
     def startFactory(self):
-        http._logDateTimeStart()
+        pass
+        #http._logDateTimeStart()
 
 
 class ProxyClient(http.HTTPClient):
@@ -174,7 +216,7 @@ class ProxyClient(http.HTTPClient):
         self.rest = rest
         if headers.has_key("proxy-connection"):
             del headers["proxy-connection"]
-        headers["connection"] = "close"
+        #headers["connection"] = "close"
         self.headers = headers
         print "command", command
         print "rest", rest
@@ -273,7 +315,6 @@ class ReverseProxyResource(proxy.ReverseProxyResource):
 
     proxyClientFactoryClass = ProxyClientFactory
 
-
     def __init__(self, host, port, path, reactor=reactor):
         """
         @param host: the host of the web server to proxy.
@@ -294,12 +335,10 @@ class ReverseProxyResource(proxy.ReverseProxyResource):
         self.path = path
         self.reactor = reactor
 
-
     def getChild(self, path, request):
         return ReverseProxyResource(
             self.host, self.port, self.path + '/' + path)
 
-
     def render(self, request):
         """
         Render a request by forwarding it to the proxied server.
@@ -322,6 +361,7 @@ class ReverseProxyResource(proxy.ReverseProxyResource):
         self.reactor.connectTCP(self.host, self.port, clientFactory)
         return server.NOT_DONE_YET
 
+
 class myHTTPPageGetter(client.HTTPPageGetter):
 
     def handleResponse(self, response):
@@ -344,8 +384,10 @@ class myHTTPPageGetter(client.HTTPPageGetter):
         # server might be stupid and not close connection. admittedly
         # the fact we do only one request per connection is also
         # stupid...
+        self.quietLoss = 1
         self.transport.loseConnection()
 
+
 class HeaderAwareHTTPClientFactory(client.HTTPClientFactory):
 
     protocol = myHTTPPageGetter
@@ -372,6 +414,7 @@ class HeaderAwareHTTPDownloader(client.HTTPDownloader):
                 # server is acting wierdly
                 self.requestedPartial = 0
 
+
 def getPage(url, contextFactory=None, *args, **kwargs):
     """Download a web page as a string.
 
@@ -391,6 +434,7 @@ def getPage(url, contextFactory=None, *args, **kwargs):
         reactor.connectTCP(host, port, factory)
     return factory.deferred
 
+
 def downloadPage(url, file, contextFactory=None, *args, **kwargs):
     """Download a web page to a file.
 
@@ -410,6 +454,7 @@ def downloadPage(url, file, contextFactory=None, *args, **kwargs):
         reactor.connectTCP(host, port, factory)
     return factory.deferred
 
+
 class StaticFile(static.File):
     """ taken from twisted.web.static and modified
         accordingly to the patch by John-Mark Gurney
@@ -535,6 +580,7 @@ class CET(tzinfo):
     def dst(self,dt):
         return timedelta(0)
 
+
 class CEST(tzinfo):
 
     def __init__(self):
@@ -550,6 +596,7 @@ class CEST(tzinfo):
     def dst(self,dt):
         return timedelta(0)
 
+
 bdates = [ datetime(1997,2,28,17,20,tzinfo=CET()),   # Sebastian Oliver
            datetime(1999,9,19,4,12,tzinfo=CEST()),   # Patrick Niklas
            datetime(2000,9,23,4,8,tzinfo=CEST()),    # Saskia Alexa
