@@ -21,7 +21,7 @@ from coherence.upnp.services.servers.av_transport_server import AVTransportServe
 
 from coherence.upnp.devices.basics import RootDeviceXML, DeviceHttpRoot, BasicDeviceMixin
 
-import louie
+import coherence.extern.louie as louie
 
 from coherence import log
 
@@ -140,4 +140,4 @@ class MediaRenderer(log.Loggable,BasicDeviceMixin):
                                                static.File(icon['url'][7:]))
 
         self.register()
-        self.info("%s %s (%s) activated" % (self.backend.name, self.device_type, self.backend))
+        self.warning("%s %s (%s) activated" % (self.backend.name, self.device_type, self.backend))
