@@ -9,7 +9,7 @@ from coherence.upnp.services.clients.av_transport_client import AVTransportClien
 
 from coherence import log
 
-import louie
+import coherence.extern.louie as louie
 
 class MediaRendererClient(log.Loggable):
     logCategory = 'mr_client'
@@ -87,12 +87,18 @@ class MediaRendererClient(log.Loggable):
         if self.detection_completed == True:
             return
         if self.rendering_control != None:
+            if not hasattr(self.rendering_control.service, 'last_time_updated'):
+                return
             if self.rendering_control.service.last_time_updated == None:
                 return
         if self.connection_manager != None:
+            if not hasattr(self.connection_manager.service, 'last_time_updated'):
+                return
             if self.connection_manager.service.last_time_updated == None:
                 return
         if self.av_transport != None:
+            if not hasattr(self.av_transport.service, 'last_time_updated'):
+                return
             if self.av_transport.service.last_time_updated == None:
                 return
         self.detection_completed = True
