@@ -12,10 +12,10 @@ from StringIO import StringIO
 import urllib
 
 from twisted.internet import task
-from twisted.internet import reactor
+from twisted.internet import reactor, defer
 from twisted.web import static
 from twisted.web import resource, server
-from twisted.web import proxy
+#from twisted.web import proxy
 from twisted.python import util
 from twisted.python.filepath import FilePath
 
@@ -25,7 +25,7 @@ from coherence import __version__
 
 from coherence.upnp.core.service import ServiceServer
 from coherence.upnp.core.utils import StaticFile
-#from coherence.upnp.core.utils import ReverseProxyResource
+from coherence.upnp.core.utils import ReverseProxyResource
 
 from coherence.upnp.services.servers.connection_manager_server import ConnectionManagerServer
 from coherence.upnp.services.servers.content_directory_server import ContentDirectoryServer
@@ -34,7 +34,7 @@ from coherence.upnp.services.servers.media_receiver_registrar_server import Fake
 
 from coherence.upnp.devices.basics import BasicDeviceMixin
 
-import louie
+import coherence.extern.louie as louie
 
 from coherence import log
 
@@ -51,6 +51,17 @@ class MSRoot(resource.Resource, log.Loggable):
         self.server = server
         self.store = store
 
+    #def delayed_response(self, resrc, request):
+    #    print "delayed_response", resrc, request
+    #    body = resrc.render(request)
+    #    print "delayed_response", body
+    #    if body == 1:
+    #        print "delayed_response not yet done"
+    #        return
+    #    request.setHeader("Content-length", str(len(body)))
+    #    request.write(response)
+    #    request.finish()
+
     def getChildWithDefault(self, path, request):
         self.info('%s getChildWithDefault, %s, %s, %s %s' % (self.server.device_type,
                                 request.method, path, request.uri, request.client))
@@ -85,9 +96,10 @@ class MSRoot(resource.Resource, log.Loggable):
             return self.import_response(path)
 
         if(headers.has_key('user-agent') and
-           headers['user-agent'].find('Xbox/') == 0 and
+           (headers['user-agent'].find('Xbox/') == 0 or      # XBox
+            headers['user-agent'].startswith("""Mozilla/4.0 (compatible; UPnP/1.0; Windows""")) and  # wmp11
            path in ['description-1.xml','description-2.xml']):
-            self.info('XBox alert, we need to simulate a Windows Media Connect server')
+            self.info('XBox/WMP alert, we need to simulate a Windows Media Connect server')
             if self.children.has_key('xbox-description-1.xml'):
                 self.msg( 'returning xbox-description-1.xml')
                 return self.children['xbox-description-1.xml']
@@ -119,9 +131,7 @@ class MSRoot(resource.Resource, log.Loggable):
 
         request.setResponseCode(404)
 
-    def getChild(self, name, request):
-        self.info('getChild %s, %s' % (name, request))
-        ch = self.store.get_by_id(name)
+    def process_child(self,ch,name,request):
         if ch != None:
             self.info('Child found', ch)
             if(request.method == 'GET' or
@@ -143,7 +153,7 @@ class MSRoot(resource.Resource, log.Loggable):
                     request.content = StringIO()
 
             if hasattr(ch, "location"):
-                if isinstance(ch.location, proxy.ReverseProxyResource):
+                if isinstance(ch.location, ReverseProxyResource):
                     self.info('getChild proxy %s to %s' % (name, ch.location.uri))
                     new_id,_,_ = self.server.connection_manager_server.add_connection('',
                                                                                 'Output',
@@ -153,7 +163,7 @@ class MSRoot(resource.Resource, log.Loggable):
                     d = request.notifyFinish()
                     d.addBoth(self.requestFinished, new_id, request)
                     request.setHeader('transferMode.dlna.org', 'Streaming')
-                    if hasattr(ch.item, 'res'):
+                    if hasattr(ch,'item') and hasattr(ch.item, 'res'):
                         if ch.item.res[0].protocolInfo is not None:
                             _,_,_,additional_info = ch.item.res[0].protocolInfo.split(':')
                             if additional_info != '*':
@@ -161,6 +171,8 @@ class MSRoot(resource.Resource, log.Loggable):
                     return ch.location
             try:
                 p = ch.get_path()
+            except TypeError:
+                return self.list_content(name, ch, request)
             except Exception, msg:
                 self.debug("error accessing items path %r" % msg)
                 self.debug(traceback.format_exc())
@@ -192,6 +204,15 @@ class MSRoot(resource.Resource, log.Loggable):
         self.info('MSRoot ch', ch)
         return ch
 
+    def getChild(self, name, request):
+        self.info('getChild %s, %s' % (name, request))
+        ch = self.store.get_by_id(name)
+        if isinstance(ch, defer.Deferred):
+            ch.addCallback(self.process_child,name,request)
+            #ch.addCallback(self.delayed_response, request)
+            return ch
+        return self.process_child(ch,name,request)
+
     def list_content(self, name, item, request):
         self.info('list_content', name, item, request)
         page = """<html><head><title>%s</title></head><body><p>%s</p>"""% \
@@ -203,36 +224,50 @@ class MSRoot(resource.Resource, log.Loggable):
             if uri[-1] != '/':
                 uri += '/'
 
-            page += """<ul>"""
-            for c in item.get_children():
-                if hasattr(c,'get_url'):
-                    path = c.get_url()
-                    self.debug('has get_url', path)
-                elif hasattr(c,'get_path'):
-                    #path = c.get_path().encode('utf-8').encode('string_escape')
-                    path = c.get_path()
-                    if isinstance(path,unicode):
-                        path = path.encode('ascii','xmlcharrefreplace')
+            def build_page(r,page):
+                print "build_page", r
+                page += """<ul>"""
+                for c in r:
+                    if hasattr(c,'get_url'):
+                        path = c.get_url()
+                        self.debug('has get_url', path)
+                    elif hasattr(c,'get_path') and c.get_path != None:
+                        #path = c.get_path().encode('utf-8').encode('string_escape')
+                        path = c.get_path()
+                        if isinstance(path,unicode):
+                            path = path.encode('ascii','xmlcharrefreplace')
+                        else:
+                            path = path.decode('utf-8').encode('ascii','xmlcharrefreplace')
+                        self.debug('has get_path', path)
                     else:
-                        path = path.decode('utf-8').encode('ascii','xmlcharrefreplace')
-                    self.debug('has get_path', path)
-                else:
-                    path = request.uri.split('/')
-                    path[-1] = str(c.get_id())
-                    path = '/'.join(path)
-                    self.debug('got path', path)
-                title = c.get_name()
-                self.debug( 'title is:', type(title))
-                try:
-                    if isinstance(title,unicode):
-                        title = title.encode('ascii','xmlcharrefreplace')
-                    else:
-                        title = title.decode('utf-8').encode('ascii','xmlcharrefreplace')
-                except (UnicodeEncodeError,UnicodeDecodeError):
-                    title = c.get_name().encode('utf-8').encode('string_escape')
-                page += '<li><a href="%s">%s</a></li>' % \
-                                    (path, title)
-            page += """</ul>"""
+                        path = request.uri.split('/')
+                        path[-1] = str(c.get_id())
+                        path = '/'.join(path)
+                        self.debug('got path', path)
+                    title = c.get_name()
+                    self.debug( 'title is:', type(title))
+                    try:
+                        if isinstance(title,unicode):
+                            title = title.encode('ascii','xmlcharrefreplace')
+                        else:
+                            title = title.decode('utf-8').encode('ascii','xmlcharrefreplace')
+                    except (UnicodeEncodeError,UnicodeDecodeError):
+                        title = c.get_name().encode('utf-8').encode('string_escape')
+                    page += '<li><a href="%s">%s</a></li>' % \
+                                        (path, title)
+                page += """</ul>"""
+                page += """</body></html>"""
+                return static.Data(page,'text/html')
+
+            children = item.get_children()
+            if isinstance(children, defer.Deferred):
+                print "list_content, we have a Deferred", children
+                children.addCallback(build_page,page)
+                #children.addErrback(....) #FIXME
+                return children
+
+            return build_page(children,page)
+
         elif( hasattr(item,'mimetype') and item.mimetype.find('image/') == 0):
             #path = item.get_path().encode('utf-8').encode('string_escape')
             path = urllib.quote(item.get_path().encode('utf-8'))
@@ -413,6 +448,7 @@ class MediaServer(log.Loggable,BasicDeviceMixin):
             return
         self.warning('backend not installed, MediaServer activation aborted - %s', msg.getErrorMessage())
         self.debug(msg)
+        del self.coherence.active_backends[str(self.uuid)]
 
     def init_complete(self, backend):
         if self.backend != backend:
@@ -482,4 +518,4 @@ class MediaServer(log.Loggable,BasicDeviceMixin):
                                                StaticFile(icon['url'][7:]))
 
         self.register()
-        self.info("%s MediaServer (%s) activated" % (self.backend.name, self.backend))
+        self.warning("%s %s (%s) activated" % (self.backend.name, self.device_type, self.backend))
