@@ -89,9 +89,10 @@ class ContentDirectoryServer(service.ServiceServer, resource.Resource,
         def got_error(r):
             return r
 
-        def process_result(result):
+        def process_result(result,total=None,found_item=None):
             if result == None:
                 result = []
+
             l = []
 
             def process_items(result, tm):
@@ -106,14 +107,30 @@ class ContentDirectoryServer(service.ServiceServer, resource.Resource,
             for i in result:
                 d = defer.maybeDeferred( i.get_item)
                 l.append(d)
-            if item == None:
-                total = 0
-            else:
+
+            if found_item != None:
+                def got_child_count(count):
+                    dl = defer.DeferredList(l)
+                    dl.addCallback(process_items, count)
+                    return dl
+
+                d = defer.maybeDeferred(found_item.get_child_count)
+                d.addCallback(got_child_count)
+
+                return d
+            elif total == None:
                 total = item.get_child_count()
+
             dl = defer.DeferredList(l)
             dl.addCallback(process_items, total)
             return dl
 
+        def proceed(result):
+            d = defer.maybeDeferred( result.get_children, StartingIndex, StartingIndex + RequestedCount)
+            d.addCallback(process_result,found_item=result)
+            d.addErrback(got_error)
+            return d
+
         try:
             root_id = ContainerID
         except:
@@ -135,32 +152,33 @@ class ContentDirectoryServer(service.ServiceServer, resource.Resource,
                             items = item[StartingIndex:]
                         else:
                             items = item[StartingIndex:StartingIndex+RequestedCount]
-                        return process_result(items)
+                        return process_result(items,total=total)
                     else:
-                        d = defer.maybeDeferred( item.get_children, StartingIndex, StartingIndex + RequestedCount)
-                        d.addCallback(process_result)
-                        d.addErrback(got_error)
-                        return d
+                        if isinstance(item,defer.Deferred):
+                            item.addCallback(proceed)
+                            return item
+                        else:
+                            return proceed(item)
 
             item = self.backend.get_by_id(root_id)
             if item == None:
-                return process_result([])
+                return process_result([],total=0)
 
-            d = defer.maybeDeferred(item.get_children, StartingIndex, StartingIndex + RequestedCount)
-            d.addCallback(process_result)
-            d.addErrback(got_error)
-
-            return d
+            if isinstance(item,defer.Deferred):
+                item.addCallback(proceed)
+                return item
+            else:
+                return proceed(item)
 
         item = self.backend.get_by_id(root_id)
         if item == None:
             return failure.Failure(errorCode(701))
 
-        d = defer.maybeDeferred(item.get_children, StartingIndex, StartingIndex + RequestedCount)
-        d.addCallback(process_result)
-        d.addErrback(got_error)
-
-        return d
+        if isinstance(item,defer.Deferred):
+            item.addCallback(proceed)
+            return item
+        else:
+            return proceed(item)
 
     def upnp_Browse(self, *args, **kwargs):
         try:
@@ -179,6 +197,11 @@ class ContentDirectoryServer(service.ServiceServer, resource.Resource,
         parent_container = None
         requested_id = None
 
+        item = None
+        total = 0
+        items = []
+
+
         if BrowseFlag == 'BrowseDirectChildren':
             parent_container = str(ObjectID)
         else:
@@ -193,7 +216,7 @@ class ContentDirectoryServer(service.ServiceServer, resource.Resource,
         def got_error(r):
             return r
 
-        def process_result(result):
+        def process_result(result,total=None,found_item=None):
             if result == None:
                 result = []
             if BrowseFlag == 'BrowseDirectChildren':
@@ -211,7 +234,20 @@ class ContentDirectoryServer(service.ServiceServer, resource.Resource,
                 for i in result:
                     d = defer.maybeDeferred( i.get_item)
                     l.append(d)
-                total = item.get_child_count()
+
+                if found_item != None:
+                    def got_child_count(count):
+                        dl = defer.DeferredList(l)
+                        dl.addCallback(process_items, count)
+                        return dl
+
+                    d = defer.maybeDeferred(found_item.get_child_count)
+                    d.addCallback(got_child_count)
+
+                    return d
+                elif total == None:
+                    total = item.get_child_count()
+
                 dl = defer.DeferredList(l)
                 dl.addCallback(process_items, total)
                 return dl
@@ -234,8 +270,16 @@ class ContentDirectoryServer(service.ServiceServer, resource.Resource,
 
             return r
 
-        total = 0
-        items = []
+        def proceed(result):
+            if BrowseFlag == 'BrowseDirectChildren':
+                d = defer.maybeDeferred( result.get_children, StartingIndex, StartingIndex + RequestedCount)
+            else:
+                d = defer.maybeDeferred( result.get_item)
+
+            d.addCallback(process_result,found_item=result)
+            d.addErrback(got_error)
+            return d
+
 
         root_id = ObjectID
 
@@ -255,33 +299,30 @@ class ContentDirectoryServer(service.ServiceServer, resource.Resource,
                             items = item[StartingIndex:]
                         else:
                             items = item[StartingIndex:StartingIndex+RequestedCount]
-                        return process_result(items)
+                        return process_result(items,total=total)
                     else:
-                        d = defer.maybeDeferred(item.get_children, StartingIndex, StartingIndex + RequestedCount)
-                        d.addCallback(process_result)
-                        d.addErrback(got_error)
-                        return d
+                        if isinstance(item,defer.Deferred):
+                            item.addCallback(proceed)
+                            return item
+                        else:
+                            return proceed(item)
 
             item = self.backend.get_by_id(root_id)
             if item == None:
-                return process_result([])
+                return process_result([],total=0)
 
-            d = defer.maybeDeferred(item.get_children, StartingIndex, StartingIndex + RequestedCount)
-            d.addCallback(process_result)
-            d.addErrback(got_error)
-
-            return d
+            if isinstance(item,defer.Deferred):
+                item.addCallback(proceed)
+                return item
+            else:
+                return proceed(item)
 
         item = self.backend.get_by_id(root_id)
         if item == None:
             return failure.Failure(errorCode(701))
 
-        if BrowseFlag == 'BrowseDirectChildren':
-            d = defer.maybeDeferred(item.get_children, StartingIndex, StartingIndex + RequestedCount)
+        if isinstance(item,defer.Deferred):
+            item.addCallback(proceed)
+            return item
         else:
-            d = defer.maybeDeferred(item.get_item)
-
-        d.addCallback(process_result)
-        d.addErrback(got_error)
-
-        return d
+            return proceed(item)
\ No newline at end of file
